# -*- coding: utf-8 -*-
# Copyright (c) 2012-2013 Infrae. All rights reserved.
# See also LICENSE.txt
# package

from silva.core.xml import registerNamespace

NS_SOURCE_URI = "http://infrae.com/namespace/silva-external-sources"
registerNamespace('silva-external-sources', NS_SOURCE_URI)
