/* Code source dialog */
.cke_dialog_contents .external_source_edit,
.cke_dialog_contents .external_source_add {
  max-height: 360px;
  overflow-y: auto;
}
.cke_dialog_contents .external_source_field {
  position: relative;
}
.cke_dialog_contents .external_source_fields label .description {
  border: 1px solid black;
  text-align: left !important;
  display: none;
  left: 10%;
  padding: 0.25em;
  position: absolute;
  top: 1.5em;
  white-space: normal;
  width: 60%;
  z-index: 2;
}
.cke_dialog_contents .external_source_fields label:hover .description {
  display: block;
}
.cke_dialog_contents .external_source_error {
  font-weigth: bold;
  color: red;
  display: block;
}
